import _regeneratorRuntime from '@babel/runtime/regenerator';
import _classCallCheck from '@babel/runtime/helpers/esm/classCallCheck';
import _createClass from '@babel/runtime/helpers/esm/createClass';
import _defineProperty from '@babel/runtime/helpers/esm/defineProperty';
import { resolve, join } from 'path';
import { copyFile, readdir, statSync } from 'fs';
import { isString, copy, camelCase } from '@forensic-js/utils';
import { config } from '../config';
import { COMMON_CONFIGS, REGEX_FIELDS } from '../constants';
import { mkDirSync, getEntryPath } from '@forensic-js/node-utils';
import { rollup } from 'rollup';
import { getRollupPlugins } from './utils';
import chalk from 'chalk';

var allExternal = function allExternal() {
  return true;
};

var log = console.log;

var Bundler =
/*#__PURE__*/
function () {
  /**
   * @param plugins array of extra plugins to be applied
   * @param config path to user defined build config or the user defined config object
   */
  function Bundler() {
    var _generalConfig$config;

    var generalConfig = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var bundlerOptions = arguments.length > 1 ? arguments[1] : undefined;

    _classCallCheck(this, Bundler);

    _defineProperty(this, "entryPath", '');

    _defineProperty(this, "config", config);

    _defineProperty(this, "generalConfig", {});

    _defineProperty(this, "bundlerOptions", void 0);

    this.entryPath = getEntryPath();
    this.generalConfig = generalConfig;
    this.config = this.resolveConfig(this.entryPath, (_generalConfig$config = generalConfig.config) !== null && _generalConfig$config !== void 0 ? _generalConfig$config : {});
    this.bundlerOptions = bundlerOptions;
  }
  /**
   * resolves the pattern into a regex object
   *@param {Array|string|RegExp} patterns - array of patterns or string pattern
   *@param {Array} regexStore - array to store regex objects
   */


  _createClass(Bundler, [{
    key: "resolveRegex",
    value: function resolveRegex(pattern) {
      if (isString(pattern)) {
        // match everything
        if (pattern === '*') {
          return new RegExp('^.*', 'i');
        } else {
          pattern = pattern.split('/').map(function (current) {
            if (current === '*') {
              return '[^/]+';
            } else if (current === '**') {
              return '.*';
            } else {
              return current;
            }
          }).join('/');
          pattern = pattern.replace(/^\/+/, '^');
          return new RegExp(pattern, 'i');
        }
      } else {
        return pattern;
      }
    }
  }, {
    key: "mergeConfig",
    value: function mergeConfig(prop, config, target) {
      var configValue = config[prop];
      var targetValue = target[prop];

      if (targetValue === undefined) {
        target[prop] = configValue;
      }
    }
    /**
     * loads the given file and returns the result
     * @param file
     */

  }, {
    key: "loadFile",
    value: function loadFile(entryPath, file) {
      try {
        return require(resolve(entryPath, file));
      } catch (ex) {
        return {};
      }
    }
    /**
     * resolves the config object
     */

  }, {
    key: "resolveConfig",
    value: function resolveConfig(entryPath, config$1) {
      var _packageFile$name,
          _this = this;

      var packageFile = this.loadFile(entryPath, 'package.json');
      var resolvedConfig = copy({}, config, config$1); //resolve module name

      resolvedConfig.moduleName = resolvedConfig.moduleName || camelCase((_packageFile$name = packageFile.name) !== null && _packageFile$name !== void 0 ? _packageFile$name : 'Unknown'); //resolve configs config

      COMMON_CONFIGS.forEach(function (prop) {
        _this.mergeConfig(prop, resolvedConfig, resolvedConfig.cjsConfig);

        _this.mergeConfig(prop, resolvedConfig, resolvedConfig.distConfig);

        _this.mergeConfig(prop, resolvedConfig, resolvedConfig.esmConfig);
      });
      /**
       * resolve regex fields
       */

      REGEX_FIELDS.forEach(function (field) {
        var values = resolvedConfig[field];
        resolvedConfig[field] = values.map(_this.resolveRegex);
      });
      return resolvedConfig;
    }
    /**
     * copies the file from the src to the destination
     */

  }, {
    key: "copyFile",
    value: function copyFile$1(src, filePath) {
      var _this2 = this;

      return new Promise(function (resolve$1, reject) {
        var dest = resolve(_this2.entryPath, filePath);
        mkDirSync(dest);
        copyFile(src, dest, function (err) {
          if (err) {
            reject(err);
          } else {
            resolve$1();
          }
        });
      });
    }
    /**
     * parses all files into module targets
     * @param modules
     * @param resolvedPath
     * @param entryFile
     * @param moduleName
     * @param currentRelativeDir
     * @param extensions
     */

  }, {
    key: "getModules",
    value: function getModules(modules, resolvedPath, entryFile, moduleName, currentRelativeDir, extensions) {
      var _this3 = this;

      return new Promise(function (resolve, reject) {
        readdir(resolvedPath, function (err, files) {
          if (err) {
            return reject(err);
          }

          var promises = [];
          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = files[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var file = _step.value;
              var filePath = join(resolvedPath, file);

              if (statSync(filePath).isDirectory()) {
                promises.push(_this3.getModules(modules, filePath, entryFile, moduleName, join(currentRelativeDir, file), extensions));
              } else {
                var firstDotPos = file.charAt(0) === '.' ? file.indexOf('.', 1) : file.indexOf('.');
                var baseName = firstDotPos > -1 ? file.substring(0, firstDotPos) : file;
                var extname = firstDotPos > -1 ? file.substring(firstDotPos) : '';
                var oldRelativePath = join(currentRelativeDir, file);
                var newRelativePath = join(currentRelativeDir, baseName + '.js');
                modules.push({
                  id: modules.length + 1,
                  ext: extname,
                  oldRelativePath: oldRelativePath,
                  newRelativePath: newRelativePath,
                  filePath: filePath,
                  name: oldRelativePath === entryFile ? moduleName : baseName,
                  isBuildFile: extensions.includes(extname)
                });
              }
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                _iterator["return"]();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }

          Promise.all(promises).then(function () {
            return resolve(modules);
          });
        });
      });
    }
  }, {
    key: "getModulesFiles",
    value: function getModulesFiles() {
      var startAt, config, modules, result, src, regexMatches, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, current, ext, isBuildFile, oldRelativePath, isTypeDefinitionFile, isAssetFile;

      return _regeneratorRuntime.async(function getModulesFiles$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              startAt = resolve(this.entryPath, this.config.srcDir);
              config = this.config;
              _context.next = 4;
              return _regeneratorRuntime.awrap(this.getModules([], startAt, config.entryFile, config.moduleName, '', config.extensions));

            case 4:
              modules = _context.sent;
              result = {
                assetFiles: [],
                buildFiles: [],
                typeDefinitionFiles: []
              };
              src = '';

              regexMatches = function regexMatches(regex) {
                return regex.test(src);
              };

              _iteratorNormalCompletion2 = true;
              _didIteratorError2 = false;
              _iteratorError2 = undefined;
              _context.prev = 11;

              for (_iterator2 = modules[Symbol.iterator](); !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                current = _step2.value;
                ext = current.ext, isBuildFile = current.isBuildFile, oldRelativePath = current.oldRelativePath;
                isTypeDefinitionFile = ext === '.d.ts';
                isAssetFile = !isTypeDefinitionFile && !isBuildFile;
                src = oldRelativePath;

                if (isTypeDefinitionFile && config.cjsConfig.enabled) {
                  result.typeDefinitionFiles.push(current);
                } else if (isAssetFile && config.assets.some(regexMatches)) {
                  result.assetFiles.push(current);
                } else if (isBuildFile && (config.include.length === 0 || config.include.some(regexMatches)) && (config.exclude.length === 0 || !config.exclude.some(regexMatches))) {
                  result.buildFiles.push(current);
                }
              }

              _context.next = 19;
              break;

            case 15:
              _context.prev = 15;
              _context.t0 = _context["catch"](11);
              _didIteratorError2 = true;
              _iteratorError2 = _context.t0;

            case 19:
              _context.prev = 19;
              _context.prev = 20;

              if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
                _iterator2["return"]();
              }

            case 22:
              _context.prev = 22;

              if (!_didIteratorError2) {
                _context.next = 25;
                break;
              }

              throw _iteratorError2;

            case 25:
              return _context.finish(22);

            case 26:
              return _context.finish(19);

            case 27:
              return _context.abrupt("return", result);

            case 28:
            case "end":
              return _context.stop();
          }
        }
      }, null, this, [[11, 15, 19, 27], [20,, 22, 26]]);
    }
  }, {
    key: "handleWarnings",
    value: function handleWarnings(warning) {
      console.log(warning.message);
    }
  }, {
    key: "handleErrors",
    value: function handleErrors(ex) {
      console.log(ex);
    }
  }, {
    key: "runBuild",
    value: function runBuild(promises, moduleFiles, config) {
      var _this4 = this;

      var assetFiles, typeDefinitionFiles, buildFiles, plugins, external, _iteratorNormalCompletion3, _didIteratorError3, _iteratorError3, _loop, _iterator3, _step3;

      return _regeneratorRuntime.async(function runBuild$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              assetFiles = moduleFiles.assetFiles, typeDefinitionFiles = moduleFiles.typeDefinitionFiles, buildFiles = moduleFiles.buildFiles;

              if (!config.enabled) {
                _context3.next = 33;
                break;
              }

              log(chalk.yellow("generating ".concat(config.format, " builds...\n")));
              plugins = getRollupPlugins(this.config, this.generalConfig, config.format === 'esm');
              external = config.format === 'iife' || config.format === 'umd' ? config.externals : allExternal;
              _iteratorNormalCompletion3 = true;
              _didIteratorError3 = false;
              _iteratorError3 = undefined;
              _context3.prev = 8;

              _loop = function _loop() {
                var _step3$value, filePath, newRelativePath, oldRelativePath, bundler, out;

                return _regeneratorRuntime.async(function _loop$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:
                        _step3$value = _step3.value, filePath = _step3$value.filePath, newRelativePath = _step3$value.newRelativePath, oldRelativePath = _step3$value.oldRelativePath;
                        _context2.prev = 1;
                        _context2.next = 4;
                        return _regeneratorRuntime.awrap(rollup({
                          input: filePath,
                          plugins: plugins,
                          external: external
                        }));

                      case 4:
                        bundler = _context2.sent;
                        out = resolve(_this4.entryPath, config.outDir, newRelativePath);
                        promises.push(bundler.write({
                          file: out,
                          format: config.format,
                          interop: config.interop,
                          sourcemap: config.sourcemap
                        }).then(function () {
                          if (_this4.bundlerOptions.generateOutputLogs) {
                            log(chalk.green("".concat(oldRelativePath, " ... ").concat(out, " \n")));
                          }
                        })["catch"](_this4.handleErrors));
                        _context2.next = 12;
                        break;

                      case 9:
                        _context2.prev = 9;
                        _context2.t0 = _context2["catch"](1);
                        console.log(_context2.t0.message);

                      case 12:
                      case "end":
                        return _context2.stop();
                    }
                  }
                }, null, null, [[1, 9]]);
              };

              _iterator3 = buildFiles[Symbol.iterator]();

            case 11:
              if (_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done) {
                _context3.next = 17;
                break;
              }

              _context3.next = 14;
              return _regeneratorRuntime.awrap(_loop());

            case 14:
              _iteratorNormalCompletion3 = true;
              _context3.next = 11;
              break;

            case 17:
              _context3.next = 23;
              break;

            case 19:
              _context3.prev = 19;
              _context3.t0 = _context3["catch"](8);
              _didIteratorError3 = true;
              _iteratorError3 = _context3.t0;

            case 23:
              _context3.prev = 23;
              _context3.prev = 24;

              if (!_iteratorNormalCompletion3 && _iterator3["return"] != null) {
                _iterator3["return"]();
              }

            case 26:
              _context3.prev = 26;

              if (!_didIteratorError3) {
                _context3.next = 29;
                break;
              }

              throw _iteratorError3;

            case 29:
              return _context3.finish(26);

            case 30:
              return _context3.finish(23);

            case 31:
              assetFiles.forEach(function (assetFile) {
                promises.push(_this4.copyFile(assetFile.filePath, resolve(_this4.entryPath, config.outDir, assetFile.oldRelativePath)));
              });
              typeDefinitionFiles.forEach(function (typeDefinitionFile) {
                promises.push(_this4.copyFile(typeDefinitionFile.filePath, resolve(_this4.entryPath, config.outDir, typeDefinitionFile.oldRelativePath)));
              });

            case 33:
            case "end":
              return _context3.stop();
          }
        }
      }, null, this, [[8, 19, 23, 31], [24,, 26, 30]]);
    }
    /**
     * runs build process for a specific module
     * @param moduleFiles
     * @param config
     */

  }, {
    key: "processModule",
    value: function processModule(moduleFiles, config) {
      var promises;
      return _regeneratorRuntime.async(function processModule$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              promises = [];
              _context4.next = 3;
              return _regeneratorRuntime.awrap(this.runBuild(promises, moduleFiles, config));

            case 3:
              return _context4.abrupt("return", Promise.all(promises).then(function () {
                return promises = null;
              }));

            case 4:
            case "end":
              return _context4.stop();
          }
        }
      }, null, this);
    }
    /**
     * runs the process
     */

  }, {
    key: "process",
    value: function process() {
      var moduleFiles;
      return _regeneratorRuntime.async(function process$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return _regeneratorRuntime.awrap(this.getModulesFiles());

            case 2:
              moduleFiles = _context5.sent;
              _context5.next = 5;
              return _regeneratorRuntime.awrap(this.processModule(moduleFiles, this.config.cjsConfig));

            case 5:
              _context5.next = 7;
              return _regeneratorRuntime.awrap(this.processModule(moduleFiles, this.config.esmConfig));

            case 7:
              _context5.next = 9;
              return _regeneratorRuntime.awrap(this.processModule(moduleFiles, this.config.distConfig));

            case 9:
            case "end":
              return _context5.stop();
          }
        }
      }, null, this);
    }
  }]);

  return Bundler;
}();

export { Bundler };
//# sourceMappingURL=Bundler.js.map
