'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _toConsumableArray = _interopDefault(require('@babel/runtime/helpers/toConsumableArray'));
var resolve = _interopDefault(require('@rollup/plugin-node-resolve'));
var commonjs = _interopDefault(require('@rollup/plugin-commonjs'));
var babel = _interopDefault(require('rollup-plugin-babel'));
var json = _interopDefault(require('@rollup/plugin-json'));
var rollupPluginTerser = require('rollup-plugin-terser');
var nodeUtils = require('@forensic-js/node-utils');
var path = _interopDefault(require('path'));
var fs = _interopDefault(require('fs'));

var resolveDependency = function resolveDependency(dir, name) {
  var dependencyPath = path.resolve(dir, 'node_modules/', name);

  try {
    var stat = fs.statSync(dependencyPath);

    if (stat && stat.isDirectory()) {
      return dependencyPath;
    }
  } catch (ex) {}

  return name;
};
/**
 * loads the given file and returns the result
 * @param file
 */


var loadFile = function loadFile(entryPath, file) {
  try {
    return require(path.resolve(entryPath, file));
  } catch (ex) {
    return {};
  }
};
var getBabelPlugins = function getBabelPlugins() {
  var extraPlugins = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var internalNodeModulesDir = arguments.length > 1 ? arguments[1] : undefined;
  var useESModules = arguments.length > 2 ? arguments[2] : undefined;
  return [[resolveDependency(internalNodeModulesDir, '@babel/plugin-transform-runtime'), {
    useESModules: useESModules
  }], resolveDependency(internalNodeModulesDir, '@babel/plugin-proposal-class-properties'), resolveDependency(internalNodeModulesDir, '@babel/plugin-proposal-object-rest-spread'), [resolveDependency(internalNodeModulesDir, '@babel/plugin-proposal-nullish-coalescing-operator')], [resolveDependency(internalNodeModulesDir, '@babel/plugin-proposal-optional-chaining')]].concat(_toConsumableArray(extraPlugins));
};
var getBabelPresets = function getBabelPresets() {
  var extraPresets = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var internalNodeModulesDir = arguments.length > 1 ? arguments[1] : undefined;
  return [[resolveDependency(internalNodeModulesDir, '@babel/preset-env'), {
    modules: false
  }], resolveDependency(internalNodeModulesDir, '@babel/preset-typescript')].concat(_toConsumableArray(extraPresets));
};
var getRollupPlugins = function getRollupPlugins(config, generalConfig) {
  var _generalConfig$babelC, _generalConfig$babelC2;

  var useESModules = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var internalNodeModulesDir = nodeUtils.getEntryPath(__dirname);
  return [resolve({
    extensions: config.extensions
  }), commonjs({
    include: 'node_modules/**'
  }), babel({
    babelrc: false,
    presets: getBabelPresets(generalConfig === null || generalConfig === void 0 ? void 0 : (_generalConfig$babelC = generalConfig.babelConfig) === null || _generalConfig$babelC === void 0 ? void 0 : _generalConfig$babelC.presets, internalNodeModulesDir),
    plugins: getBabelPlugins(generalConfig === null || generalConfig === void 0 ? void 0 : (_generalConfig$babelC2 = generalConfig.babelConfig) === null || _generalConfig$babelC2 === void 0 ? void 0 : _generalConfig$babelC2.plugins, internalNodeModulesDir, useESModules),
    exclude: 'node_modules/**',
    extensions: config.extensions,
    runtimeHelpers: true
  }), json(), config.uglify && nodeUtils.isProdEnv() && rollupPluginTerser.terser()].concat(_toConsumableArray(config.plugins));
};

exports.getBabelPlugins = getBabelPlugins;
exports.getBabelPresets = getBabelPresets;
exports.getRollupPlugins = getRollupPlugins;
exports.loadFile = loadFile;
//# sourceMappingURL=utils.js.map
